@@ -1,54 +1,13 @@
 ### Continuous Mode of a distribution
-old_mode <- function(x) {
-      if(length(x)==2) return(median(x))
-      d <-tryCatch(density(as.numeric(x), na.rm = TRUE, n = 128, from = min(x), to = max(x)), error = function(e) (median(x) + mean(fivenum(x)[2:4]))/2)
-      tryCatch(d$x[which.max(d$y)], error = function(e) d)
-}
-
-
-### Continuous mode if data previously sorted
-mode <- function(x){
-  x <- as.numeric(x)
-  l <- length(x)
-  if(l <= 3) return(median(x))
-  if(length(unique(x))==1) return(x[1])
-  x_s <- x[order(x)]
-  range <- abs(x_s[l]-x_s[1])
-  if(range==0) return(x[1])
-
-  z <- MESS::bin(x_s, range/128, origin = x_s[1], missinglast = FALSE)
-  lz <- length(z$counts)
-  max_z <- z$counts==max(z$counts[3:(lz-2)])
-  
-  if(sum(max_z)>1){
-    z_ind <- 3:(lz-2)
-    z_names <- seq(x_s[1], x_s[l], z$width)
-    return(sum(z_names[z_ind] * z$counts[z_ind] )/sum(z$counts[z_ind]))
-  } else {
-    z_c <- which.max(z$counts)
-    
-    z_ind <- max(1, (z_c - 1)):min(l,(z_c + 1))
-    z_names <- seq(x_s[1], x_s[l], z$width)
-    return(sum(z_names[z_ind] * z$counts[z_ind] )/sum(z$counts[z_ind]))
-  }
-}
-
+mode <- function(x) NNS.mode(x, discrete = FALSE, multi = FALSE)
 
 
 ### Classification Mode of a distribution
-mode_class <- function(x){
-  x <- na.omit(x)
-  ux <- unique(x)
-  ux[which.max(tabulate(match(x, ux)))]
-}
+mode_class <- function(x) NNS.mode(x, discrete = TRUE, multi = FALSE)
 
 
-
-
-### Central Tendency
-old_gravity <- function(x) (median(x) + mode(x) + mean(fivenum(x)[2:4]))/3
-
-gravity <- function(x){
+### Gravity of a distribution
+gravity <- function(x, discrete = FALSE){
   l <- length(x)
   if(l <= 3) return(median(x))
   if(length(unique(x))==1) return(x[1])
@@ -63,13 +22,13 @@ gravity <- function(x){
 
   z <- MESS::bin(x_s, range/128, origin = x_s[1], missinglast = FALSE)
   lz <- length(z$counts)
-  max_z <- z$counts==max(z$counts[3:(lz-2)])
+  max_z <- z$counts==max(z$counts)
   
   if(sum(max_z)>1){
-    z_ind <- 3:(lz-2)
+    z_ind <- 1:lz
   } else {
     z_c <- which.max(z$counts)
-    z_ind <- max(1, (z_c - 1)):min(l,(z_c + 1))
+    z_ind <- max(1, (z_c - 1)):min(lz,(z_c + 1))
   }
 
   z_names <- seq(x_s[1], x_s[l], z$width)
@@ -78,12 +37,12 @@ gravity <- function(x){
   mu <- sum(x)/l
   
   res <- (q2 + m + mu + mean(c(q1, q2, q3)))/4
-  if(is.na(res)) return(q2) else return(res)
+  if(is.na(res)) final <- q2 else final <- res
+  if(discrete) return(ifelse(final%%1 < .5, floor(final), ceiling(final))) else return(final)
 } 
 
 
-gravity_class <- function(x) mode_class(x)
-
+gravity_class <- function(x) gravity(x, discrete = TRUE)
 
 
 ### Factor to dummy variable
