@@ -10,7 +10,6 @@
 #' @param dim.red.method options: ("cor", "NNS.dep", "NNS.caus", "all", "equal", \code{numeric vector}, NULL) method for determining synthetic X* coefficients.  Selection of a method automatically engages the dimension reduction regression.  The default is \code{NULL} for full multivariate regression.  \code{(dim.red.method = "NNS.dep")} uses \link{NNS.dep} for nonlinear dependence weights, while \code{(dim.red.method = "NNS.caus")} uses \link{NNS.caus} for causal weights.  \code{(dim.red.method = "cor")} uses standard linear correlation for weights.  \code{(dim.red.method = "all")} averages all methods for further feature engineering.  \code{(dim.red.method = "equal")} uses unit weights.  Alternatively, user can specify a numeric vector of coefficients.
 #' @param tau options("ts", NULL); \code{NULL}(default) To be used in conjunction with \code{(dim.red.method = "NNS.caus")} or \code{(dim.red.method = "all")}.  If the regression is using time-series data, set \code{(tau = "ts")} for more accurate causal analysis.
 #' @param type \code{NULL} (default).  To perform a classification, set to \code{(type = "CLASS")}.  Like a logistic regression, it is not necessary for target variable of two classes e.g. [0, 1].
-#' @param inference logical; \code{FALSE} (default) For inferential tasks, otherwise \code{inference = FALSE} is faster for predictive tasks.
 #' @param point.est a numeric or factor vector with compatible dimensions to \code{x}.  Returns the fitted value \code{y.hat} for any value of \code{x}.
 #' @param location Sets the legend location within the plot, per the \code{x} and \code{y} co-ordinates used in base graphics \link{legend}.
 #' @param return.values logical; \code{TRUE} (default), set to \code{FALSE} in order to only display a regression plot and call values as needed.
@@ -22,7 +21,7 @@
 #' @param n.best integer; \code{NULL} (default) Sets the number of nearest regression points to use in weighting for multivariate regression at \code{sqrt(# of regressors)}.  \code{(n.best = "all")} will select and weight all generated regression points.  Analogous to \code{k} in a
 #' \code{k Nearest Neighbors} algorithm.  Different values of \code{n.best} are tested using cross-validation in \link{NNS.stack}.
 #' @param noise.reduction the method of determining regression points options: ("mean", "median", "mode", "off"); In low signal:noise situations,\code{(noise.reduction = "mean")}  uses means for \link{NNS.dep} restricted partitions, \code{(noise.reduction = "median")} uses medians instead of means for \link{NNS.dep} restricted partitions, while \code{(noise.reduction = "mode")}  uses modes instead of means for \link{NNS.dep} restricted partitions.  \code{(noise.reduction = "off")} uses an overall central tendency measure for partitions.
-#' @param dist options:("L1", "L2", "DTW", "FACTOR") the method of distance calculation; Selects the distance calculation used. \code{dist = "L2"} (default) selects the Euclidean distance and \code{(dist = "L1")} seclects the Manhattan distance; \code{(dist = "DTW")} selects the dynamic time warping distance; \code{(dist = "FACTOR")} uses a frequency.
+#' @param dist options:("L1", "L2", "FACTOR") the method of distance calculation; Selects the distance calculation used. \code{dist = "L2"} (default) selects the Euclidean distance and \code{(dist = "L1")} selects the Manhattan distance; \code{(dist = "FACTOR")} uses a frequency.
 #' @param ncores integer; value specifying the number of cores to be used in the parallelized  procedure. If NULL (default), the number of cores to be used is equal to the number of cores of the machine - 1.
 #' @param multivariate.call Internal argument for multivariate regressions.
 #' @param point.only Internal argument for abbreviated output.
@@ -130,7 +129,6 @@ NNS.reg = function (x, y,
                     stn = .95,
                     dim.red.method = NULL, tau = NULL,
                     type = NULL,
-                    inference = FALSE,
                     point.est = NULL,
                     location = "top",
                     return.values = TRUE,
@@ -142,7 +140,7 @@ NNS.reg = function (x, y,
                     dist = "L2", ncores = NULL,
                     point.only = FALSE,
                     multivariate.call = FALSE){
-
+  
   oldw <- getOption("warn")
   options(warn = -1)
   
@@ -158,6 +156,7 @@ NNS.reg = function (x, y,
   if(any(class(x)%in%c("tbl","data.table"))) x <- as.data.frame(x)
   
   n <- length(y)
+  original.x <- x
   
   if(n < 2000) ncores <- 1
   if(!is.null(dim(x))){
@@ -305,6 +304,7 @@ NNS.reg = function (x, y,
         if(num_cores > 1){
           cl <- parallel::makeCluster(num_cores)
           doParallel::registerDoParallel(cl)
+          invisible(data.table::setDTthreads(1))
         }
         
         if(is.null(original.names)){
@@ -361,7 +361,6 @@ NNS.reg = function (x, y,
               tau <- "cs"
             }
             x.star.coef <- numeric()
-            
             cause <- list()
             
             cause <- foreach(i = 1:dim(x)[2], .packages = c("NNS", "data.table"))%dopar%{
@@ -379,9 +378,10 @@ NNS.reg = function (x, y,
             if(is.null(tau)) tau <- "cs"
             
             x.star.coef.1 <- numeric()
+            cause <- list()
             
             cause <- foreach(i = 1:dim(x)[2], .packages = c("NNS", "data.table"))%dopar%{
-              return(Uni.caus(y, x[,i], tau = tau, plot = FALSE))
+              return(Uni.caus(x[,i], y, tau = tau, plot = FALSE))
             }
             
             x.star.coef.1 <- unlist(cause)
@@ -390,9 +390,17 @@ NNS.reg = function (x, y,
             x.star.coef.3[is.na(x.star.coef.3)] <- 0
             x.star.coef.2 <- x.star.dep
             x.star.coef.2[is.na(x.star.coef.2)] <- 0
-            x.star.coef <- Rfast::rowmeans(cbind(x.star.coef.1, x.star.coef.2, x.star.coef.3))
+            x.star.coef.4 <- rep(1, ncol(x))
+            x.star.coef <- apply(cbind(x.star.coef.1, x.star.coef.2, x.star.coef.3, x.star.coef.4), 1, function(x) mode(x)) 
             x.star.coef[is.na(x.star.coef)] <- 0
           }
+        
+        if(num_cores > 1){    
+          parallel::stopCluster(cl)
+          registerDoSEQ()
+          invisible(data.table::setDTthreads(0, throttle = NULL))
+        }
+          
           
           if(!is.numeric(dim.red.method) && dim.red.method == "equal")  x.star.coef <- rep(1, ncol(x))
           
@@ -413,6 +421,8 @@ NNS.reg = function (x, y,
             x.star.coef[x.star.coef == 0] <- preserved.coef
           }
           
+          xn <- sum( abs( x.star.coef) > 0)
+          
           if(is.numeric(dim.red.method)) DENOMINATOR <- sum(dim.red.method) else DENOMINATOR <- sum( abs( x.star.coef) > 0)
           
           synthetic.x.equation.coef <- data.table::data.table(Variable = colnames.list, Coefficient = x.star.coef)
@@ -428,43 +438,32 @@ NNS.reg = function (x, y,
               points.norm <- apply(points.norm, 2, function(b) (b - min(b)) / ifelse((max(b) - min(b))==0, 1, (max(b) - min(b))))
             }
             if(is.null(np) || np==1){
-              new.point.est <- sum(points.norm[1,] * x.star.coef) / sum( abs( x.star.coef) > 0)
+              new.point.est <- sum(points.norm[1,] * x.star.coef) / xn
               
             } else {
               point.est2 <- points.norm[1:np,]
               new.point.est <- apply(point.est2, 1, function(i) as.numeric(as.vector(i)[!is.na(i)|!is.nan(i)] %*% x.star.coef[!is.na(i)|!is.nan(i)])
-                                     / sum( abs( x.star.coef) > 0))
+                                     / xn)
             }
             
             point.est <- new.point.est
             
           }
           
-          
-          x <- Rfast::rowsums(x.star.matrix / sum( abs( x.star.coef) > 0), parallel = TRUE)
-          
-          x.star <- data.table::data.table(x.star = x)
+          x <- Rfast::rowsums(x.star.matrix / sum( abs( x.star.coef) > 0), parallel = FALSE)
+          x.star <- data.table::data.table(x)
           
         }
-        
-        
-        if(num_cores>1){
-          parallel::stopCluster(cl)
-          registerDoSEQ()
-        }
-        
-        
-      } # Multivariate Not NULL type
+       } # Multivariate Not NULL type
       
     } # Univariate
     
   } # Multivariate
   
-  
+
   x.label <- names(x)
   if(is.null(x.label)) x.label <- "x"
-  
-  
+   
   dependence <- tryCatch(NNS.dep(x, y, print.map = FALSE, asym = TRUE, ncores = 1)$Dependence, error = function(e) .1)
   dependence <- dependence^2
   dependence[is.na(dependence)] <- 0
@@ -473,11 +472,10 @@ NNS.reg = function (x, y,
   dep.reduced.order <- max(1, ifelse(is.null(order), rounded_dep, order))
   
   if(!is.null(order)) dep.reduced.order <- order
-    
+
   if(dependence == 1 || dep.reduced.order == "max"){
     if(is.null(order)) dep.reduced.order <- "max"
     part.map <- NNS.part(x, y, order = dep.reduced.order, obs.req = 0)
-    part.map1 <- part.map
   } else {
     if(is.null(type)){
       noise.reduction2 <- ifelse(noise.reduction=="mean", "off", noise.reduction)
@@ -487,44 +485,81 @@ NNS.reg = function (x, y,
     
     if(dep.reduced.order == "max"){
       part.map <- NNS.part(x, y, order = dep.reduced.order, obs.req = 0)
-      part.map1 <- part.map
-    } else if(inference){
-      part.map1 <- NNS.part(x, y, noise.reduction = noise.reduction2, order = dep.reduced.order, type = "XONLY", obs.req = 0)
-      part.map <- NNS.part(c(x, part.map1$regression.points$x), c(y, part.map1$regression.points$y), noise.reduction = noise.reduction2, order = dep.reduced.order, type = "XONLY", obs.req = 0)
+    } else {
+      part.map <- NNS.part(x, y, noise.reduction = noise.reduction2, order = dep.reduced.order, type = "XONLY", obs.req = 0)
+      
+      part.map1 <- NNS.part(c(x, part.map$regression.points$x), c(y, part.map$regression.points$y), noise.reduction = noise.reduction2, order = dep.reduced.order, type = "XONLY", obs.req = 0)
       part.map2 <- NNS.part(c(x, part.map$regression.points$x, part.map1$regression.points$x), c(y, part.map$regression.points$y, part.map1$regression.points$y), noise.reduction = noise.reduction2, order = dep.reduced.order, type = "XONLY", obs.req = 0)
       part.map3 <- NNS.part(c(x, part.map$regression.points$x, part.map1$regression.points$x, part.map2$regression.points$x), c(y, part.map$regression.points$y, part.map1$regression.points$y, part.map2$regression.points$y), noise.reduction = noise.reduction2, order =  dep.reduced.order, type =  "XONLY", obs.req = 0)
       part.map4 <- NNS.part(c(x, part.map$regression.points$x, part.map1$regression.points$x, part.map2$regression.points$x, part.map3$regression.points$x), c(y, part.map$regression.points$y, part.map1$regression.points$y, part.map2$regression.points$y, part.map3$regression.points$y), noise.reduction = noise.reduction2, order =  dep.reduced.order, type =  "XONLY", obs.req = 0)
-      
-      part.map <- NNS.part(c(part.map$regression.points$x, part.map1$regression.points$x, part.map2$regression.points$x, part.map3$regression.points$x, part.map4$regression.points$x),
-                           c(part.map$regression.points$y, part.map1$regression.points$y, part.map2$regression.points$y, part.map3$regression.points$y, part.map4$regression.points$y),
-                           noise.reduction = noise.reduction2, order = dep.reduced.order, type = "XONLY", obs.req = 0)
-      
-      i <- 1
-      while(length(part.map$regression.points$x) > length(x)){
+
+      if(Reduce(identical, lapply(list(part.map$regression.points$x, part.map1$regression.points$x, part.map2$regression.points$x, part.map3$regression.points$x, part.map4$regression.points$x), length)) &&
+         Reduce(identical, lapply(list(part.map$regression.points$y, part.map1$regression.points$y, part.map2$regression.points$y, part.map3$regression.points$y, part.map4$regression.points$y), length))){
+        
+        part.map$regression.points$x <- apply(cbind(c(part.map$regression.points$x, part.map1$regression.points$x, part.map2$regression.points$x, part.map3$regression.points$x, part.map4$regression.points$x)),1, function(x) gravity(x))
+        part.map$regression.points$y <- apply(cbind(c(part.map$regression.points$y, part.map1$regression.points$y, part.map2$regression.points$y, part.map3$regression.points$y, part.map4$regression.points$y)),1, function(x) gravity(x))
+      } else {
         part.map <- NNS.part(c(part.map$regression.points$x, part.map1$regression.points$x, part.map2$regression.points$x, part.map3$regression.points$x, part.map4$regression.points$x),
                              c(part.map$regression.points$y, part.map1$regression.points$y, part.map2$regression.points$y, part.map3$regression.points$y, part.map4$regression.points$y),
-                             noise.reduction = noise.reduction2, order =  max(1, dep.reduced.order - i), type = "XONLY", obs.req = 0)
-        i <- i + 1
+                             noise.reduction = noise.reduction2, order = dep.reduced.order, type = "XONLY", obs.req = 0)
       }
-      if(length(part.map1$regression.points$x) == 0){
-        part.map <- NNS.part(x, y, type =  "XONLY", noise.reduction = noise.reduction2, order = min( nchar(part.map1$dt$quadrant)), obs.req = 0)
-        part.map1 <- part.map
+      
+      if(length(part.map$regression.points$x) > length(y)){
+        i <- 0
+        while(length(part.map$regression.points$x) > length(y)){
+          part.map <- NNS.part(c(part.map$regression.points$x, part.map1$regression.points$x, part.map2$regression.points$x, part.map3$regression.points$x, part.map4$regression.points$x),
+                               c(part.map$regression.points$y, part.map1$regression.points$y, part.map2$regression.points$y, part.map3$regression.points$y, part.map4$regression.points$y),
+                               noise.reduction = noise.reduction2, order =  max(c(1, (dep.reduced.order - i))), type = "XONLY", obs.req = 0)
+          i <- i + 1
+        }
+      }
+      
+      if(length(part.map$regression.points$x) == 0){
+        part.map <- NNS.part(x, y, type =  "XONLY", noise.reduction = noise.reduction2, order = min( nchar(part.map$dt$quadrant)), obs.req = 0)
       }
-    } else {
-      part.map <- NNS.part(x, y, type =  "XONLY", order = dep.reduced.order, obs.req = 0, noise.reduction = noise.reduction2)
-      part.map1 <- part.map
     }
   }
   
-  
+  if(length(part.map$dt$y) > length(y)){
+    part.map$dt$x <- pmax(min(x), pmin(part.map$dt$x, max(x)))
+    part.map$dt[, y := gravity(y), by = "x"]
+    data.table::setkey(part.map$dt, x)
+    part.map$dt <- unique(part.map$dt, by = "x")
+  }
+
   Regression.Coefficients <- data.frame(matrix(ncol = 3))
   colnames(Regression.Coefficients) <- c('Coefficient', 'X Lower Range', 'X Upper Range')
   
   regression.points <- part.map$regression.points[,.(x,y)]
+
   regression.points$x <- pmin(max(x), pmax(regression.points$x, min(x)))
-  
+ 
   data.table::setkey(regression.points,x)
+  regression.points <- regression.points[, y := gravity(y), by = "x"]
+  regression.points <- unique(regression.points)
   
+  
+  if(type!="class" || is.null(type)){
+    central_rows <- c(floor(median(1:nrow(regression.points))), ceiling(median(1:nrow(regression.points))))
+    central_x <- regression.points[central_rows,]$x
+    ifelse(length(unique(central_rows))>1, central_y <- gravity(y[x>=central_x[1] & x<=central_x[2]]), central_y <- regression.points[central_rows[1],]$y)
+    central_x <- gravity(central_x)
+    med.rps <- t(c(central_x, central_y))
+  } else {
+    med.rps <- t(c(NA, NA))
+  }
+ 
+  regression.points <- data.table::rbindlist(list(regression.points,data.table::data.table(do.call(rbind, list(med.rps)))), use.names = FALSE)
+  
+  regression.points <- regression.points[complete.cases(regression.points),]
+  regression.points <- regression.points[ , .(x,y)]
+  data.table::setkey(regression.points, x, y)
+  
+  ### Consolidate possible duplicated points
+  regression.points <- regression.points[, y := gravity(y), by = "x"]
+  regression.points <- unique(regression.points)
+  
+
   if(dependence < 1){
     min.range <- min(regression.points$x)
     max.range <- max(regression.points$x)
@@ -615,7 +650,6 @@ NNS.reg = function (x, y,
       }
     }
     
-    
     ### Endpoints
     max.rps <- t(c(max(x), mean(x.max)))
     min.rps <- t(c(min(x), mean(x0)))
@@ -625,49 +659,19 @@ NNS.reg = function (x, y,
     min.rps <- t(c(min(x), y[x == min(x)][1]))
   }
   
-  if(type!="class" || is.null(type)){
-    central_rows <- c(floor(median(1:nrow(regression.points))), ceiling(median(1:nrow(regression.points))))
-    central_x <- regression.points[central_rows,]$x
-    
-    ifelse(length(unique(central_rows))>1, central_y <- gravity(y[x>=central_x[1] & x<=central_x[2]]), central_y <- regression.points[central_rows[1],]$y)
-    central_x <- mean(central_x)
-    med.rps <- t(c(central_x, central_y))
-  } else {
-    med.rps <- t(c(NA, NA))
-  }
+
   
   regression.points <- data.table::rbindlist(list(regression.points,data.table::data.table(do.call(rbind, list(min.rps, max.rps, med.rps )))), use.names = FALSE)
   
-  
   regression.points <- regression.points[complete.cases(regression.points),]
   regression.points <- regression.points[ , .(x,y)]
   data.table::setkey(regression.points, x, y)
-  regression.points <- unique(regression.points)
   
   ### Consolidate possible duplicated points
-  if(any(duplicated(regression.points$x))){
-    if(noise.reduction == "off"){
-      regression.points <- regression.points[, lapply(.SD, gravity), .SDcols = 2, by = .(x)]
-    }
-    
-    if(noise.reduction == "mean"){
-      regression.points <- regression.points[, lapply(.SD, mean), .SDcols = 2, by = .(x)]
-    }
-    
-    if(noise.reduction == "median"){
-      regression.points <- regression.points[, lapply(.SD, median), .SDcols = 2, by = .(x)]
-    }
-    
-    if(noise.reduction == "mode"){
-      regression.points <- regression.points[, lapply(.SD, mode), .SDcols = 2, by = .(x)]
-    }
-    if(noise.reduction == "mode_class"){
-      regression.points <- regression.points[, lapply(.SD, mode_class), .SDcols = 2, by = .(x)]
-    } 
-  }
-  
-  
+  regression.points <- regression.points[, y := gravity(y), by = "x"]
+  regression.points <- unique(regression.points)
   
+
   if(dim(regression.points)[1] > 1){
     rise <- regression.points[ , 'rise' := y - data.table::shift(y)]
     run <- regression.points[ , 'run' := x - data.table::shift(x)]
@@ -701,7 +705,7 @@ NNS.reg = function (x, y,
   
   ### Fitted Values
   p <- length(unlist(regression.points[ , 1]))
-  
+ 
   
   if(is.na(Regression.Coefficients[1, Coefficient])){
     Regression.Coefficients[1, Coefficient := Regression.Coefficients[2, Coefficient] ]
@@ -712,8 +716,7 @@ NNS.reg = function (x, y,
   
   coef.interval <- findInterval(x, Regression.Coefficients[ , (X.Lower.Range)], left.open = FALSE)
   reg.interval <- findInterval(x, regression.points[, x], left.open = FALSE)
-  
-  
+
   
   if(is.discrete(order) || ifelse(is.null(order), FALSE, ifelse(order >= length(y), TRUE, FALSE))){
     estimate <- y
@@ -745,11 +748,11 @@ NNS.reg = function (x, y,
   if(!is.null(type)){
     if(type=="class") estimate <- pmin(max(y), pmax(min(y), ifelse(estimate%%1 < .5, floor(estimate), ceiling(estimate))))
   }
-  
-  fitted <- data.table::data.table(x = part.map1$dt$x,
-                                   y = part.map1$dt$y,
+
+  fitted <- data.table::data.table(x = x,
+                                   y = original.y,
                                    y.hat = estimate,
-                                   NNS.ID = part.map1$dt$quadrant)
+                                   NNS.ID = part.map$dt$quadrant)
   
   colnames(fitted) <- gsub("y.hat.V1", "y.hat", colnames(fitted))
   
@@ -762,13 +765,14 @@ NNS.reg = function (x, y,
   y.fitted <- fitted[ , y.hat]
   
   gradient <- Regression.Coefficients$Coefficient[findInterval(fitted$x, Regression.Coefficients$X.Lower.Range)]
-  
+
   fitted <- cbind(fitted, gradient)
   fitted$residuals <- original.y - fitted$y.hat
   
   if(dependence < stn && mean(c(length(unique(diff(x))), length(unique(x)))) > .33*length(x)){
     bias <- fitted
-    data.table::setkey(bias, x)
+    
+    data.table::setkey(bias, NNS.ID)
     
     bias <- bias[, gravity(residuals)*-1, by = gradient]
     
@@ -865,10 +869,10 @@ NNS.reg = function (x, y,
     if(type=="class") estimate <- pmin(max(y), pmax(min(y), ifelse(estimate%%1 < 0.5, floor(estimate), ceiling(estimate))))
   }
   
-  fitted <- data.table::data.table(x = part.map1$dt$x,
-                                   y = part.map1$dt$y,
+  fitted <- data.table::data.table(x = x,
+                                   y = original.y,
                                    y.hat = estimate,
-                                   NNS.ID = part.map1$dt$quadrant)
+                                   NNS.ID = part.map$dt$quadrant)
   
   colnames(fitted) <- gsub("y.hat.V1", "y.hat", colnames(fitted))
   
@@ -885,28 +889,29 @@ NNS.reg = function (x, y,
   fitted <- cbind(fitted, gradient)
   fitted$residuals <-  original.y - fitted$y.hat
   
-  
   if(!is.null(type)){
     if(type=="class") Prediction.Accuracy <- (length(y) - sum( abs( round(y.fitted) - (y)) > 0)) / length(y) else Prediction.Accuracy <- NULL
   } else {
     Prediction.Accuracy <- NULL
   }
-  
-  
-  if((sum((fitted[ , y.hat] - mean(y)) * (y - mean(y))) ^ 2) == 0){
+
+  R2num <- sum((y.fitted - mean(original.y))*(original.y - mean(original.y)))^ 2
+  R2den <- sum((y.fitted - mean(original.y)) ^ 2 )* sum((original.y - mean(original.y)) ^ 2)
+
+  if(R2den == 0){
     R2 <- 1
   } else {
-    R2 <- (sum((fitted[ , y.hat] - mean(y)) * (y - mean(y))) ^ 2) / (sum((y - mean(y)) ^ 2) * sum((fitted[ , y.hat] - mean(y)) ^ 2))
+    R2 <- max(0, min(1, R2num / R2den ))
   }
   
   
   ###Standard errors estimation
-    fitted[, `:=` ( 'standard.errors' = sqrt( sum((y.hat - y) ^ 2) / ( max(1,(.N - 1))) ) ), by = gradient]
-    
+  fitted[, `:=` ( 'standard.errors' = sqrt( sum((y.hat - y) ^ 2) / ( max(1,(.N - 1))) ) ), by = gradient]
+  
   
   ###Plotting and regression equation
   if(plot){
-    r2.leg <- bquote(bold(R ^ 2 == .(format(R2, digits = 4))))
+    if(!is.null(type) && type=="class") r2.leg <- paste("Accuracy: ", format(Prediction.Accuracy, digits = 4)) else  r2.leg <- bquote(bold(R ^ 2 == .(format(R2, digits = 4))))
     xmin <- min(c(point.est, x))
     xmax <- max(c(point.est, x))
     ymin <- min(c(point.est.y, y, fitted$y.hat, regression.points$y))
@@ -919,15 +924,13 @@ NNS.reg = function (x, y,
     }
     
     if(is.numeric(confidence.interval)){
-      fitted[, `:=` ( 'conf.int.pos' = abs(UPM.VaR(1 - confidence.interval, degree = 1, y[y>y.hat])) + y.hat ), by = gradient]
-      fitted[, `:=` ( 'conf.int.neg' = y.hat - abs(LPM.VaR(1 - confidence.interval, degree = 1, y[y<y.hat]))  ), by = gradient]
+      fitted[, `:=` ( 'conf.int.pos' = UPM.VaR(1 - confidence.interval, degree = 1, y) ), by = gradient]
+      fitted[, `:=` ( 'conf.int.neg' = LPM.VaR(1 - confidence.interval, degree = 1, y) ), by = gradient]
       
       pval <- 1 - confidence.interval
-      se.max <- max(na.omit(fitted[ , y.hat + (qt(p = 1 - (pval / 2), df = max(1, .N - 1) ) * standard.errors)]))
-      se.min <- min(na.omit(fitted[, y.hat - (qt(p = 1 - (pval / 2), df = max(1, .N - 1) ) * standard.errors)]))
       
       plot(x, y, xlim = c(xmin, xmax),
-           ylim = c(min(c(se.min, ymin)), max(c(se.max,ymax))),
+           ylim = c(min(c(fitted$conf.int.neg, ymin)), max(c(fitted$conf.int.pos,ymax))),
            col ='steelblue', main = paste(paste0("NNS Order = ", plot.order), sep = "\n"),
            xlab = if(!is.null(original.columns)){
              if(original.columns > 1){
@@ -939,8 +942,8 @@ NNS.reg = function (x, y,
            ylab = y.label, mgp = c(2.5, 0.5, 0),
            cex.lab = 1.5, cex.main = 2)
       
-      points(na.omit(fitted[ , .(x, conf.int.pos)]), col = 'pink', pch = 19)
-      points(na.omit(fitted[ , .(x, conf.int.neg)]), col = 'pink', pch = 19)
+      points(fitted$x, na.omit(fitted$conf.int.pos), col = 'pink', pch = 19)
+      points(fitted$x, na.omit(fitted$conf.int.neg), col = 'pink', pch = 19)
     } else {
       plot(x, y, xlim = c(xmin, xmax), ylim = c(ymin, ymax),col = 'steelblue', main = paste(paste0("NNS Order = ", plot.order), sep = "\n"),
            xlab = if(!is.null(original.columns)){
