@@ -1,6 +1,9 @@
 Uni.caus <- function(x, y, tau, plot = TRUE){
 
-  xy <- NNS.norm(cbind(x, y), linear = FALSE)
+  if(tau=="cs") tau <- 0
+  if(tau=="ts") tau <- 3
+
+  xy <- NNS.norm(cbind(x, y), linear = FALSE, chart.type = NULL)
 
   NNS.x <- unlist(xy[ , 1])
   NNS.y <- unlist(xy[ , 2])
@@ -48,7 +51,7 @@ Uni.caus <- function(x, y, tau, plot = TRUE){
 
 
   ## Correlation of Normalized Variables
-  rho.x.y <- NNS.dep(y.norm.to.x, x.norm.to.y, asym = TRUE)$Dependence
+  rho.x.y <- NNS.dep(y.norm.to.x, x.norm.to.y, asym = TRUE, ncores = 1)$Dependence
 
   Causation.x.given.y <- P.x.given.y * rho.x.y
 
